@@ -2,11 +2,11 @@
  * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson.
  * Copyright (c) 2007-2008, The Tor Project, Inc. */
 /* See LICENSE for licensing information */
-/* $Id: compat.h 15804 2008-07-09 15:23:35Z nickm $ */
+/* $Id: compat.h 17255 2008-11-12 01:10:21Z sjm217 $ */
 
 #ifndef __COMPAT_H
 #define __COMPAT_H
-#define COMPAT_H_ID "$Id: compat.h 15804 2008-07-09 15:23:35Z nickm $"
+#define COMPAT_H_ID "$Id: compat.h 17255 2008-11-12 01:10:21Z sjm217 $"
 
 #include "orconfig.h"
 #include "torint.h"
@@ -463,7 +463,7 @@ void set_uint32(char *cp, uint32_t v) ATTR_NONNULL((1));
 typedef unsigned long rlim_t;
 #endif
 int set_max_file_descriptors(rlim_t limit, int *max);
-int switch_id(const char *user, const char *group);
+int switch_id(const char *user);
 #ifdef HAVE_PWD_H
 char *get_user_homedir(const char *username);
 #endif
