@@ -1,13 +1,13 @@
 /* Copyright 2002 Christopher Clark */
 /* Copyright 2005 Nick Mathewson */
 /* See license at end. */
-/* $Id: ht.h 8776 2006-10-20 16:22:53Z nickm $ */
+/* $Id: ht.h 13665 2008-02-21 21:57:47Z nickm $ */
 
 /* Based on ideas by Christopher Clark and interfaces from Niels Provos. */
 
 #ifndef __HT_H
 #define __HT_H
-#define HT_H_ID "$Id: ht.h 8776 2006-10-20 16:22:53Z nickm $"
+#define HT_H_ID "$Id: ht.h 13665 2008-02-21 21:57:47Z nickm $"
 
 #define HT_HEAD(name, type)                                             \
   struct name {                                                         \
@@ -65,6 +65,7 @@ ht_improve_hash(unsigned h)
   return h;
 }
 
+#if 0
 /** Basic string hash function, from Java standard String.hashCode(). */
 static INLINE unsigned
 ht_string_hash(const char *s)
@@ -77,11 +78,25 @@ ht_string_hash(const char *s)
   }
   return h;
 }
+#endif
+
+/** Basic string hash function, from Python's str.__hash__() */
+static INLINE unsigned
+ht_string_hash(const char *s)
+{
+  unsigned h;
+  const unsigned char *cp = (const unsigned char *)s;
+  h = *cp << 7;
+  while (*cp) {
+    h = (1000003*h) ^ *cp++;
+  }
+  /* This conversion truncates the length of the string, but that's ok. */
+  h ^= (unsigned)(cp-(const unsigned char*)s);
+  return h;
+}
 
 #define _HT_SET_HASH(elm, field, hashfn)        \
-  do {                                          \
-    elm->field.hte_hash = hashfn(elm);          \
-  } while (0)
+    (elm)->field.hte_hash = hashfn(elm)
 
 #define HT_FOREACH(x, name, head)                 \
   for ((x) = HT_START(name, head);                \
@@ -91,7 +106,7 @@ ht_string_hash(const char *s)
 #define HT_PROTOTYPE(name, type, field, hashfn, eqfn)                   \
   int name##_HT_GROW(struct name *ht, unsigned min_capacity);           \
   void name##_HT_CLEAR(struct name *ht);                                \
-  int _##name##_HT_REP_IS_BAD(struct name *ht);                         \
+  int _##name##_HT_REP_IS_BAD(const struct name *ht);                   \
   static INLINE void                                                    \
   name##_HT_INIT(struct name *head) {                                   \
     head->hth_table_length = 0;                                         \
@@ -119,11 +134,12 @@ ht_string_hash(const char *s)
   /* Return a pointer to the element in the table 'head' matching 'elm', \
    * or NULL if no such element exists */                               \
   static INLINE struct type *                                           \
-  name##_HT_FIND(struct name *head, struct type *elm)                   \
+  name##_HT_FIND(const struct name *head, struct type *elm)             \
   {                                                                     \
     struct type **p;                                                    \
+    struct name *h = (struct name *) head;                              \
     _HT_SET_HASH(elm, field, hashfn);                                   \
-    p = _##name##_HT_FIND_P(head, elm);                                 \
+    p = _##name##_HT_FIND_P(h, elm);                                    \
     return p ? *p : NULL;                                               \
   }                                                                     \
   /* Insert the element 'elm' into the table 'head'.  Do not call this  \
@@ -272,7 +288,7 @@ ht_string_hash(const char *s)
     805306457, 1610612741                                               \
   };                                                                    \
   static unsigned name##_N_PRIMES =                                     \
-    sizeof(name##_PRIMES)/sizeof(name##_PRIMES[0]);                     \
+    (unsigned)(sizeof(name##_PRIMES)/sizeof(name##_PRIMES[0]));         \
   /* Expand the internal table of 'head' until it is large enough to    \
    * hold 'size' elements.  Return 0 on success, -1 on allocation       \
    * failure. */                                                        \
@@ -349,7 +365,7 @@ ht_string_hash(const char *s)
   /* Debugging helper: return false iff the representation of 'head' is \
    * internally consistent. */                                          \
   int                                                                   \
-  _##name##_HT_REP_IS_BAD(struct name *head)                            \
+  _##name##_HT_REP_IS_BAD(const struct name *head)                      \
   {                                                                     \
     unsigned n, i;                                                      \
     struct type *elm;                                                   \
@@ -383,6 +399,31 @@ ht_string_hash(const char *s)
     return 0;                                                           \
   }
 
+/** Implements an over-optimized "find and insert if absent" block;
+ * not meant for direct usage by typical code, or usage outside the critical
+ * path.*/
+#define _HT_FIND_OR_INSERT(name, field, hashfn, head, eltype, elm, var, y, n) \
+  {                                                                     \
+    struct name *_##var##_head = head;                                  \
+    eltype **var;                                                       \
+    if (!_##var##_head->hth_table ||                                    \
+        _##var##_head->hth_n_entries >= _##var##_head->hth_load_limit)  \
+      name##_HT_GROW(_##var##_head, _##var##_head->hth_n_entries+1);     \
+    _HT_SET_HASH((elm), field, hashfn);                                \
+    var = _##name##_HT_FIND_P(_##var##_head, (elm));                    \
+    if (*var) {                                                         \
+      y;                                                                \
+    } else {                                                            \
+      n;                                                                \
+    }                                                                   \
+  }
+#define _HT_FOI_INSERT(field, head, elm, newent, var)       \
+  {                                                         \
+    newent->field.hte_hash = (elm)->field.hte_hash;         \
+    *var = newent;                                          \
+    ++((head)->hth_n_entries);                              \
+  }
+
 /*
  * Copyright 2005, Nick Mathewson.  Implementation logic is adapted from code
  * by Cristopher Clark, retrofit to allow drop-in memory management, and to
