@@ -2,9 +2,9 @@
  * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson.
  * Copyright (c) 2007-2008, The Tor Project, Inc. */
 /* See LICENSE for licensing information */
-/* $Id: dirvote.c 14587 2008-05-12 04:29:07Z nickm $ */
+/* $Id: dirvote.c 17254 2008-11-12 00:58:10Z sjm217 $ */
 const char dirvote_c_id[] =
-  "$Id: dirvote.c 14587 2008-05-12 04:29:07Z nickm $";
+  "$Id: dirvote.c 17254 2008-11-12 00:58:10Z sjm217 $";
 
 #define DIRVOTE_PRIVATE
 #include "or.h"
@@ -1089,8 +1089,8 @@ networkstatus_add_detached_signatures(networkstatus_t *target,
         memcpy(target_voter->signing_key_digest, src_voter->signing_key_digest,
                DIGEST_LEN);
         target_voter->signature_len = src_voter->signature_len;
-        target_voter->good_signature = 1;
-        target_voter->bad_signature = 0;
+        target_voter->good_signature = src_voter->good_signature;
+        target_voter->bad_signature = src_voter->bad_signature;
       } else {
         log_info(LD_DIR, "Not adding signature from %s", voter_identity);
       }
@@ -1872,12 +1872,17 @@ dirvote_add_signatures_to_pending_consensus(
                                             sigs, msg_out);
   log_info(LD_DIR,"Added %d signatures to consensus.", r);
 
-  if (r >= 0) {
+  if (r >= 1) {
     char *new_detached =
       networkstatus_get_detached_signatures(pending_consensus);
     const char *src;
     char *dst, *dst_end;
-    size_t new_consensus_len =
+    size_t new_consensus_len;
+    if (!new_detached) {
+      *msg_out = "No signatures to add";
+      goto err;
+    }
+    new_consensus_len =
       strlen(pending_consensus_body) + strlen(new_detached) + 1;
     pending_consensus_body = tor_realloc(pending_consensus_body,
                                          new_consensus_len);
