@@ -3,9 +3,9 @@
  * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson.
  * Copyright (c) 2007-2008, The Tor Project, Inc. */
 /* See LICENSE for licensing information */
-/* $Id: main.c 15182 2008-06-13 04:18:26Z arma $ */
+/* $Id: main.c 17260 2008-11-12 14:26:38Z kloesing $ */
 const char main_c_id[] =
-  "$Id: main.c 15182 2008-06-13 04:18:26Z arma $";
+  "$Id: main.c 17260 2008-11-12 14:26:38Z kloesing $";
 
 /**
  * \file main.c
@@ -1113,8 +1113,10 @@ run_scheduled_events(time_t now)
   circuit_close_all_marked();
 
   /** 7. And upload service descriptors if necessary. */
-  if (has_completed_circuit && !we_are_hibernating())
+  if (has_completed_circuit && !we_are_hibernating()) {
     rend_consider_services_upload(now);
+    rend_consider_descriptor_republication();
+  }
 
   /** 8. and blow away any connections that need to die. have to do this now,
    * because if we marked a conn for close and left its socket -1, then
