@@ -3,9 +3,9 @@
  * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson.
  * Copyright (c) 2007-2008, The Tor Project, Inc. */
 /* See LICENSE for licensing information */
-/* $Id: relay.c 15185 2008-06-13 05:12:27Z arma $ */
+/* $Id: relay.c 17342 2008-11-20 22:21:31Z arma $ */
 const char relay_c_id[] =
-  "$Id: relay.c 15185 2008-06-13 05:12:27Z arma $";
+  "$Id: relay.c 17342 2008-11-20 22:21:31Z arma $";
 
 /**
  * \file relay.c
@@ -751,8 +751,11 @@ connection_edge_process_end_not_open(
             ttl = (int)ntohl(get_uint32(cell->payload+RELAY_HEADER_SIZE+5));
           else
             ttl = -1;
-          client_dns_set_addressmap(conn->socks_request->address, addr,
-                                    conn->chosen_exit_name, ttl);
+
+          if (!(get_options()->ClientDNSRejectInternalAddresses &&
+                                           is_internal_IP(addr, 0)))
+            client_dns_set_addressmap(conn->socks_request->address, addr,
+                                      conn->chosen_exit_name, ttl);
         }
         /* check if he *ought* to have allowed it */
         if (exitrouter &&
