@@ -1,9 +1,9 @@
 /* Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson.
  * Copyright (c) 2007-2008, The Tor Project, Inc. */
 /* See LICENSE for licensing information */
-/* $Id: rendcommon.c 13605 2008-02-20 00:30:41Z arma $ */
+/* $Id: rendcommon.c 17260 2008-11-12 14:26:38Z kloesing $ */
 const char rendcommon_c_id[] =
-  "$Id: rendcommon.c 13605 2008-02-20 00:30:41Z arma $";
+  "$Id: rendcommon.c 17260 2008-11-12 14:26:38Z kloesing $";
 
 /**
  * \file rendcommon.c
@@ -32,6 +32,10 @@ rend_service_descriptor_free(rend_service_descriptor_t *desc)
       rend_intro_point_free(intro););
     smartlist_free(desc->intro_nodes);
   }
+  if (desc->successful_uploads) {
+    SMARTLIST_FOREACH(desc->successful_uploads, char *, c, tor_free(c););
+    smartlist_free(desc->successful_uploads);
+  }
   tor_free(desc);
 }
 
@@ -884,6 +888,7 @@ rend_cache_store(const char *desc, size_t desc_len, int published)
   if (!published && strmap_get_lc(rend_cache, key)) {
     log_info(LD_REND, "We already have a v2 descriptor for service %s.",
              safe_str(query));
+    rend_service_descriptor_free(parsed);
     return -1;
   }
   /* report novel publication to statistics */
